
#include <kallisti/abi/ta.h>
#include "jpeglib.h"
#include "image.h"

// size of n*n poly
uint32 POLYWH=64;


extern abi_ta_t *ta;

uint32 h=0,y=0,n=0,m=0,polysdeep=0;

// fills polys array of image with jpeg scan line
void scan_to_image(char *buffer, uint32 row_stride, image *img)
{ 
    uint16 ourbuffer[10000]={0};
    uint32 i;
    uint32 polysacross=0;

    for(i=0; i<row_stride/3; i++){
        ourbuffer[i] = ((buffer[i*3]>>3)<<11) + ((buffer[i*3+1]>>2)<<5) + (buffer[i*3+2]>>3);
	}
	
	if(n == POLYWH ){
		polysdeep += img->horiz_polys;
		n=0;
	}

    for(i=0;i<img->horiz_polys;i++){    	 
    	memcpy(img->polys[polysdeep+polysacross].txrtmp + (y%POLYWH)*POLYWH, &ourbuffer[polysacross*POLYWH], 2*POLYWH);
    	polysacross++;
    }
    
	n++;
	y++; 
}

// reads jpeg, modified jpeg-test example
int JPEG_to_image(char * filename, image *img)
{
    struct jpeg_decompress_struct cinfo;
    
    struct jpeg_error_mgr jerr;
    uint32 i,k,l,x;

    FILE * infile;
    JSAMPARRAY buffer;
    uint32 row_stride;
    
    if ((infile = fopen(filename, "rb")) == NULL) {

	return 0;
    }
    h=y=n=m=polysdeep=0;

    
    cinfo.err = jpeg_std_error(&jerr);
   
    jpeg_create_decompress(&cinfo);
    
    jpeg_stdio_src(&cinfo, infile);
    
    (void) jpeg_read_header(&cinfo, TRUE);
  
    (void) jpeg_start_decompress(&cinfo);
   
    row_stride = cinfo.output_width * cinfo.output_components;
  
    buffer = (*cinfo.mem->alloc_sarray)
	((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);
    
	y=h=i=0;
 
  	while(i<row_stride/3){
    	i+=POLYWH;
    }
    img->horiz_polys=i/POLYWH;
    
    i=0;
    while(i<cinfo.output_height){
    	i+=POLYWH;
    }
    img->vert_polys=i/POLYWH;
    
    for(i=0;i<img->vert_polys;i++){
		for(k=0;k<img->horiz_polys;k++){
			if(y>POLYWH+POLYWH*h) h++;
			img->polys[i*img->horiz_polys+k].disc_txr=ta->txr_allocate(POLYWH*POLYWH*2);
			img->polys[i*img->horiz_polys+k].txrtmp = ta->txr_map(img->polys[i*img->horiz_polys+k].disc_txr);
			for(x=0; x<POLYWH*POLYWH; x++){
				img->polys[i*img->horiz_polys+k].txrtmp[x]=0;
			}
		}
	}
    
    l=0;
    
    while (cinfo.output_scanline < cinfo.output_height) {
	
		(void) jpeg_read_scanlines(&cinfo, buffer, 1);
	
    	scan_to_image(buffer[0], row_stride, img);

	}
    
    (void) jpeg_finish_decompress(&cinfo);
   
    jpeg_destroy_decompress(&cinfo);
   
    fclose(infile);
   
    return 1;
}

// draw single  poly
void draw_poly(double x, double y, double xwidth, double yheight, int bifilt, image_poly *pol, double a)
{ 
	poly_hdr_t hdr;
	vertex_ot_t	v;
    if( bifilt==1){
		ta->poly_hdr_txr(&hdr, TA_OPAQUE, TA_RGB565, POLYWH, POLYWH, pol->disc_txr, TA_BILINEAR_FILTER);
	}else{
		ta->poly_hdr_txr(&hdr, TA_OPAQUE, TA_RGB565, POLYWH, POLYWH, pol->disc_txr, TA_NO_FILTER);
	}
	ta->commit_poly_hdr(&hdr);

	v.flags = TA_VERTEX_NORMAL;
	v.x = x;
	v.y = y;
	v.z = 5.0f;
	v.u = 0.0f;
	v.v = 1;
	v.a = 1.0f; v.r = v.g = v.b = a;
	v.oa = v.or = v.og = v.ob = 0.0f;
	ta->commit_vertex(&v, sizeof(v));

	v.x = x;
	v.y = y-yheight;
	v.v = 0.0f;
	v.u = 0.0f;
	ta->commit_vertex(&v, sizeof(v));

	v.x = x+xwidth;
	v.y = y;
	v.u = 1.0f;
	v.v = 1.0f;
	ta->commit_vertex(&v, sizeof(v));

	v.flags = TA_VERTEX_EOL;
	v.x = x+xwidth;
	v.y = y-yheight;
	v.v = 0.0f;
	v.u = 1.0f;
	ta->commit_vertex(&v, sizeof(v));
}

// draws image
void draw_image(double x, double y, double totalwidth, double totalheight, int bifilt, image *img, float a)
{
   	uint32 i,k;
   	for(i=1;i <= img->vert_polys;i++){
		for(k=0;k<img->horiz_polys;k++){
			draw_poly(x+k*(totalwidth/img->horiz_polys),y-i*(totalheight/img->vert_polys), (totalwidth/img->horiz_polys), (totalheight/img->vert_polys), bifilt, &img->polys[(img->vert_polys-i)*img->horiz_polys+k], a);
		}
	}
		
}

void draw_image_full(int bifilt, image *img, float a)
{
	float ratio, newh, neww;
	if( (img->horiz_polys*64.0f)/640.0f >= (img->vert_polys*64.0f)/480.0f ){
		ratio=640.0f/(img->horiz_polys*64.0f);
		neww=640.0f;
		newh=ratio*img->vert_polys*64.0f;
		draw_image( 0.0f, 500.0f-((480.0f-newh)/2.0f), neww, newh, 0, img, a);
	}else{
		ratio=480.0f/(img->vert_polys*64.0f);
		newh=480.0f;
		neww=ratio*img->horiz_polys*64.0f;
		draw_image( 0.0f+((640.0f-neww)/2.0f), 500.0f, neww, newh, 0, img, a);
	}
}
